# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Data Importing from BRdata ``itemfile.txt`` file
"""

from __future__ import unicode_literals

import logging
from decimal import Decimal

from rattail.db import importing
from rattail.gpc import GPC
from rattail_brdata.parsers import parse_itemfile
from rattail_brdata.util import normalize_upc


log = logging.getLogger(__name__)


def import_from_brdata(config, session, models=None, providers=None, warnings=False, progress=None):
    """
    Import pertinent data from a BRdata export file.
    """
    if not models:
        models = [
            'Product',
            'ProductCost',
            ]
    if providers is None:
        providers = globals()

    for name in models:

        # Create the provider object.
        provider_name = '{0}Provider'.format(name)
        if hasattr(providers, 'get'):
            Provider = providers.get(provider_name)
        else:
            Provider = getattr(providers, provider_name, None)
        assert Provider, "No provider class found for model: {0}".format(repr(name))
        provider = Provider(config, session)

        # Import some data!
        data = provider.get_data(progress=progress)
        affected = provider.importer.import_data(data, provider.supported_fields, provider.key, progress=progress)
        log_func = log.warning if warnings and affected else log.info
        log_func("added or updated {0} {1} records".format(affected, name))


class BRdataProvider(importing.DataProvider):
    """
    Base class for all BRdata data providers.
    """

    def get_source_data(self, progress=None):
        path = self.config.require('rattail_brdata', 'source.itemfile')
        return parse_itemfile(path, progress=progress)

    def normalize_upc(self, upc):
        return normalize_upc(upc)


class ProductProvider(BRdataProvider):
    """
    Provider for product data from BRdata export.
    """
    importer_class = importing.models.ProductImporter
    key = 'upc'
    supported_fields = [
        'upc',
        'brand_name',
        'description',
        'size',
        'regular_price_price',
        'subdepartment_number',
        'category_number',
        'report_code',
        'case_pack',
        ]

    def normalize(self, data):
        upc = self.normalize_upc(data['UPC Code'])

        category = data['Category/Sub Code'].strip()
        category = int(category) if category else None

        return {
            'upc':                      upc,
            'brand_name':               data['Brand'],
            'description':              data['Description'],
            'size':                     data['Size'],
            'regular_price_price':      Decimal(data['Retail']),
            'subdepartment_number':     int(data['Dept. Code']), # sic
            'category_number':          category,
            'report_code':              self.normalize_report_code(data),
            'case_pack':                int(data['Pack']),
            }

    def normalize_report_code(self, data):
        report_code = data['Report Code'].strip()
        if report_code.isdigit():
            report_code = int(report_code)
        else:
            if report_code:
                log.warning("product has invalid report code: {0}".format(data))
            report_code = None
        return report_code


class ProductCostProvider(BRdataProvider):
    """
    Provider for product cost data from BRdata export.
    """
    importer_class = importing.models.ProductCostImporter
    key = ('product_upc', 'vendor_id')
    supported_fields = [
        'product_upc',
        'vendor_id',
        'code',
        'case_cost',
        'preferred',
        ]

    def normalize(self, data):
        upc = self.normalize_upc(data['UPC Code'])

        vendor_id = data['Vendor'].strip()

        if upc and vendor_id:
            return {
                'product_upc':      upc,
                'vendor_id':        vendor_id,
                'code':             data['Vendor Item'] if vendor_id else None,
                'case_cost':        Decimal(data['Base/Case']) if vendor_id else None,
                }
