# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2016 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Data Importing from BRdata ``lblout.txt`` file
"""

from __future__ import unicode_literals, absolute_import

import re
import datetime
import decimal
import logging

from rattail.db import model, newimporting as importing
from rattail.gpc import GPC
from rattail.time import localtime, make_utc
from rattail.util import OrderedDict

from rattail_brdata.parsers import parse_lblout
from rattail_brdata.util import normalize_upc


log = logging.getLogger(__name__)


class LbloutImportHandler(importing.ImportHandler):
    """
    Import handler for BRdata ``lblout.txt`` file.
    """

    def get_importers(self):
        importers = OrderedDict()
        importers['Product'] = ProductImporter
        importers['ProductCost'] = ProductCostImporter
        return importers


class LbloutImporter(importing.Importer):
    """
    Base class for all BRdata ``lblout.txt`` data providers.
    """

    def get_source_data(self):
        path = self.config.require('rattail_brdata', 'source.lblout')
        return parse_lblout(path, progress=self.progress)

    def normalize_upc(self, upc):
        return normalize_upc(upc)

    def normalize_sale_ends(self, sale_ends):
        sale_ends = sale_ends.strip() or None
        if sale_ends:
            sale_ends = datetime.datetime.strptime(sale_ends, '%m/%d/%y')
            sale_ends = localtime(self.config, sale_ends)
            sale_ends = sale_ends.replace(hour=23, minute=59)
            return make_utc(sale_ends)


class ProductImporter(LbloutImporter, importing.model.ProductImporter):
    """
    Importer for product data from BRdata 'lblout.txt' file.
    """
    key = 'upc'
    supported_fields = [
        'upc',
        'brand_name',
        'description',
        'unit_size',
        'size',
        'regular_price_price',
        'sale_price_price',
        'sale_price_starts',
        'sale_price_ends',
        'department_uuid',
        'subdepartment_number',
        'category_number',
        'report_code',
        'case_pack',
        'weighed',
        'discountable',
    ]

    def setup(self):
        super(ProductImporter, self).setup()
        self.subdept_x_dept = self.cache_model(model.Subdepartment, key='number',
                                               normalizer=lambda s: s.department_uuid)
        self.normalized_upcs = set()

    def normalize_source_record(self, data):
        upc = self.normalize_upc(data['UPC'])
        if upc in self.normalized_upcs:
            log.warning("duplicate UPC detected: {}".format(upc.pretty()))
        else:
            self.normalized_upcs.add(upc)

        unit_size = data['Unit Size'].strip() or None
        if unit_size:
            unit_size = decimal.Decimal(unit_size)

        subdept_number = int(data['Department']) # sic

        category = data['Category-Sub'].strip()
        category = int(category) if category else None

        report_code = data['Report Code'].strip()
        if report_code.isdigit():
            report_code = int(report_code)
        else:
            if report_code:
                log.warning("product has invalid report code: {0}".format(data))
            report_code = None

        # Prices less than $1 show as e.g. 99¢.
        price = data['Retail'].strip()
        price = price.lstrip('$')
        if price.endswith('¢'):
            price = '.{0}'.format(price.rstrip('¢'))
        try:
            price = decimal.Decimal(price) or None
        except decimal.InvalidOperation:
            log.warning("row has invalid price: {0}".format(repr(data)))
            price = None

        # Prices less than $1 show as e.g. 99¢.
        sale_price = data['Price'].strip()
        sale_price = sale_price.lstrip('$')
        if sale_price.endswith('¢'):
            sale_price = '.{0}'.format(sale_price.rstrip('¢'))
        try:
            sale_price = decimal.Decimal(sale_price) or None
        except decimal.InvalidOperation:
            log.warning("row has invalid price: {0}".format(repr(data)))
            sale_price = None

        # Determine if sale price is currently active; if it is not then we
        # will declare None for all sale fields.
        sale_starts = self.make_utc(data['From Date'])
        sale_ends = self.normalize_sale_ends(data['To Date'])
        if sale_price:
            active = False
            if sale_starts and sale_ends:
                if sale_starts <= self.now <= sale_ends:
                    active = True
            elif sale_starts:
                if sale_starts <= self.now:
                    active = True
            elif sale_ends:
                if self.now <= sale_ends:
                    active = True
            else:
                active = True
            if not active or (sale_price == price and not sale_ends):
                sale_price = None
                sale_starts = None
                sale_ends = None

        return {
            'upc':                      upc,
            'brand_name':               data['Brand'],
            'description':              data['Description'],
            'unit_size':                unit_size,
            'size':                     data['Size'].strip(),
            'regular_price_price':      price,
            'sale_price_price':         sale_price,
            'sale_price_starts':        sale_starts if sale_price else None,
            'sale_price_ends':          sale_ends if sale_price else None,
            'department_uuid':          self.subdept_x_dept.get(subdept_number),
            'subdepartment_number':     subdept_number,
            'category_number':          category,
            'report_code':              report_code,
            'case_pack':                int(data['Pack']),
            'weighed':                  data['Scale Item'].strip() == 'Y',
            'discountable':             data['Discountable'].strip() == 'Y',
        }

    def make_utc(self, date):
        date = date.strip() or None
        if date:
            time = datetime.datetime.strptime(date, '%m/%d/%y')
            return make_utc(localtime(self.config, time))


class ProductCostImporter(LbloutImporter, importing.model.ProductCostImporter):
    """
    Importer for product cost data from BRdata 'lblout.txt' file.
    """
    key = ('product_upc', 'vendor_id')
    supported_fields = [
        'product_upc',
        'vendor_id',
        'code',
        'case_cost',
        'preferred',
    ]

    def normalize_source_record(self, data):
        upc = self.normalize_upc(data['UPC'])
        vendor_id = data['Vendor No'].strip()
        if upc and vendor_id:
            return {
                'product_upc':      upc,
                'vendor_id':        vendor_id,
                'code':             data['Vendor Item'].strip() if vendor_id else None,
                'case_cost':        decimal.Decimal(data['Cost/Cs']) if vendor_id else None,
            }
