# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Data Importing from BRdata ``90days.csv`` movement file
"""

from __future__ import unicode_literals

import datetime
import logging
from decimal import Decimal

from rattail.db import importing
from rattail.gpc import GPC
from rattail_brdata.parsers import parse_movement
from rattail_brdata.util import normalize_upc
from rattail.time import localtime, make_utc


log = logging.getLogger(__name__)


def import_from_brdata(config, session, models=None, providers=None, warnings=False, progress=None):
    """
    Import pertinent data from a BRdata ``90days.csv`` movement file.
    """
    if not models:
        models = [
            'Product',
            ]
    if providers is None:
        providers = globals()

    for name in models:

        # Create the provider object.
        provider_name = '{0}Provider'.format(name)
        if hasattr(providers, 'get'):
            Provider = providers.get(provider_name)
        else:
            Provider = getattr(providers, provider_name, None)
        assert Provider, "No provider class found for model: {0}".format(repr(name))
        provider = Provider(config, session)

        # Import some data!
        data = provider.get_data(progress=progress)
        affected = provider.importer.import_data(data, provider.supported_fields, provider.key, progress=progress)
        log_func = log.warning if warnings and affected else log.info
        log_func("added or updated {0} {1} records".format(affected, name))


class MovementProvider(importing.DataProvider):
    """
    Base class for all BRdata ``lblout.txt`` data providers.
    """

    def get_source_data(self, progress=None):
        path = self.config.require('rattail_brdata', 'source.movement')
        return parse_movement(path, progress=progress)

    def normalize_upc(self, upc):
        return normalize_upc(upc)


class ProductProvider(MovementProvider):
    """
    Provider for product data from BRdata 'lblout.txt' file.
    """
    importer_class = importing.models.ProductImporter
    key = 'upc'
    supported_fields = [
        'upc',
        'brand',
        'description',
        'size',
        'regular_price_price',
        'subdepartment_number',
        'category_number',
        'report_code',
        'case_pack',
        ]

    def normalize(self, data):
        upc = self.normalize_upc(data['UPC/PLU'])

        price = data['Price'].strip() or None
        if price:
            price = Decimal(price)

        category = data['Category/Sub'].strip() or None
        if category:
            category = int(category)

        report_code = data['Report Code'].strip() or None
        if report_code:
            report_code = int(report_code)

        return {
            'upc':                      upc,
            'brand':                    data['Brand'],
            'description':              data['Long Description'],
            'size':                     data['Size'],
            'regular_price_price':      price,
            'subdepartment_number':     int(data['Department']), # sic
            'category_number':          category,
            'report_code':              report_code,
            'case_pack':                int(data['Pack']),
            'last_sold':                self.normalize_last_sold(),
            }

    def normalize_last_sold(self, data):
        last_sold = data['Date'].strip() or None
        if last_sold:
            last_sold = datetime.datetime.combine(
                datetime.datetime.strptime(last_sold, '%Y%m%d').date(),
                datetime.time(0))
            last_sold = localtime(self.config, last_sold)
            last_sold = make_utc(last_sold)
        return last_sold
